/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
 *
 *  Use of this source code is governed by a BSD-style license that can
 *  be found in the License.html file in the root of the source tree.
 */

//  Created by Max Pozdeev on 08.02.12.

#import <Cocoa/Cocoa.h>


@interface TreeOutline : NSOutlineView {

}

- (IBAction)copy:(id)sender;

@end
