mkdir _Automated
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\easyviewwidget.h -o_Automated\moc_easyviewwidget.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\export.h -o_Automated\moc_export.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\mainwindow.h -o_Automated\moc_mainwindow.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\preferences.h -o_Automated\moc_preferences.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\easyviewwidget.h -o_Automated\moc_easyviewwidget.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\export.h -o_Automated\moc_export.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\about.h -o_Automated\moc_about.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\sheetview.h -o_Automated\moc_sheetview.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\columneditsheet.h -o_Automated\moc_columneditsheet.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\editsheet.h -o_Automated\moc_editsheet.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\configtreetext.h -o_Automated\moc_configtreetext.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\editconfigtreetext.h -o_Automated\moc_editconfigtreetext.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\editcustom.h -o_Automated\moc_editcustom.cpp
..\..\..\..\Qt\bin\moc ..\..\..\Source\GUI\Qt\bottombar.h -o_Automated\moc_bottombar.cpp
..\..\..\..\Qt\bin\rcc ..\..\..\Source\Resource\Resources.qrc -o _Automated\qrc_Resources.cpp
..\..\..\..\Qt\bin\uic ..\..\..\Source\GUI\Qt\export.ui -o _Automated\ui_export.h
..\..\..\..\Qt\bin\uic ..\..\..\Source\GUI\Qt\mainwindow.ui -o _Automated\ui_mainwindow.h
..\..\..\..\Qt\bin\uic ..\..\..\Source\GUI\Qt\preferences.ui -o _Automated\ui_preferences.h
..\..\..\..\Qt\bin\uic ..\..\..\Source\GUI\Qt\export.ui -o _Automated\ui_export.h
..\..\..\..\Qt\bin\uic ..\..\..\Source\GUI\Qt\about.ui -o _Automated\ui_about.h
..\..\..\..\Qt\bin\uic ..\..\..\Source\GUI\Qt\sheetview.ui -o _Automated\ui_sheetview.h
..\..\..\..\Qt\bin\uic ..\..\..\Source\GUI\Qt\editsheet.ui -o _Automated\ui_editsheet.h
..\..\..\..\Qt\bin\uic ..\..\..\Source\GUI\Qt\editconfigtreetext.ui -o _Automated\ui_editconfigtreetext.h
..\..\..\..\Qt\bin\uic ..\..\..\Source\GUI\Qt\editcustom.ui -o _Automated\ui_editcustom.h
..\..\..\..\Qt\bin\uic ..\..\..\Source\GUI\Qt\bottombar.ui -o _Automated\ui_bottombar.h
