<!--
  Copyright (c) MediaArea.net SARL. All Rights Reserved.

      Use of this source code is governed by a BSD-style license that can
      be found in the License.html file in the root of the source tree.
-->
<Application
    x:Class="MediaInfo.App"
    xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
    xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
    RequestedTheme="Light">
    <Application.Resources>
        <Style TargetType="ContentDialog" x:Key="LargeDialog">
            <Setter Property="MaxHeight" Value="640" />
            <Setter Property="MaxWidth" Value="768" />
        </Style>
    </Application.Resources>
</Application>
