#include <winresrc.h>

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 23,04,0,0
 PRODUCTVERSION 23,04,0,0
 FILEFLAGSMASK 0x17L
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x40004L
 FILETYPE 0x2L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "CompanyName", "MediaArea.net"
            VALUE "FileDescription", "MediaInfo"
            VALUE "FileVersion", "17.05.0.0"
            VALUE "InternalName", "MediaInfo - Qt GUI"
            VALUE "LegalCopyright", "MediaArea.net"
            VALUE "OriginalFilename", "MediaInfo.exe"
            VALUE "ProductName", "MediaInfo"
            VALUE "ProductVersion", "17.05.0.0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END

//Icons
aaaaaaaa    ICON "../../../Source/Resource/Image/MediaInfo_TinyOnly.ico"
