/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
*
*  Use of this source code is governed by a BSD-style license that can
*  be found in the License.html file in the root of the source tree.
*/

#pragma once

#include <collection.h>
#include <ppltasks.h>

#include "App.xaml.h"

#include "AppCore.h"

#include "Report.h"
#include "ReportDataSource.h"
#include "ReportViewModel.h"

#include "SheetViewHelpers.h"

#include "DataGrid.xaml.h"
#include "SheetEditFieldControl.xaml.h"
#include "SheetEditDialog.xaml.h"

#include "EasyView.xaml.h"
#include "SheetView.xaml.h"
#include "HtmlView.xaml.h"

#include "AboutPage.xaml.h"
#include "ReportPage.xaml.h"
#include "MainPage.xaml.h"