/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
 *
 *  Use of this source code is governed by a BSD-style license that can
 *  be found in the License.html file in the root of the source tree.
 */

#ifndef TRANSLATE_H
#define TRANSLATE_H

#include <QtCore/QString>
#include <QtCore/QDebug>
QString Tr(const char* string, const char * disambiguation = 0, int n = -1);

#endif // TRANSLATE_H
