/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
 *
 *  Use of this source code is governed by a BSD-style license that can
 *  be found in the License.html file in the root of the source tree.
 */

//  Created by Max Pozdeev on 20.06.11.

#import <Cocoa/Cocoa.h>


@interface NSString (NSString_wchar)

+ (NSString*) stringFromWCHAR:(const wchar_t*) inStr;
- (const wchar_t *) WCHARString;

@end
