<!--
  Copyright (c) MediaArea.net SARL. All Rights Reserved.

      Use of this source code is governed by a BSD-style license that can
      be found in the License.html file in the root of the source tree.
-->
<UserControl
    x:Class="MediaInfo.SheetEditFieldControl"
    xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
    xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
    xmlns:local="using:MediaInfo"
    xmlns:d="http://schemas.microsoft.com/expression/blend/2008"
    xmlns:mc="http://schemas.openxmlformats.org/markup-compatibility/2006"
    mc:Ignorable="d">
    <Grid>
        <Grid.ColumnDefinitions>
            <ColumnDefinition />
            <!-- Separator -->
            <ColumnDefinition Width="5" />
            <ColumnDefinition Width="Auto" />
            <!-- Separator -->
            <ColumnDefinition Width="5" />
            <ColumnDefinition />
            <!-- Separator -->
            <ColumnDefinition Width="5" />
            <ColumnDefinition Width="Auto"/>
            <!-- Separator -->
            <ColumnDefinition Width="5" />
            <ColumnDefinition />
        </Grid.ColumnDefinitions>
        <Grid.RowDefinitions>
            <RowDefinition Height="Auto" />
            <RowDefinition Height="Auto" />
            <RowDefinition Height="Auto" />
        </Grid.RowDefinitions>
        <TextBlock Grid.Column="0" Grid.Row="0" Text="Name" FontSize="16" Foreground="Gray" />
        <TextBlock Grid.Column="2" Grid.Row="0" Text="Width (1-9)" FontSize="16" Foreground="Gray" />
        <TextBlock Grid.Column="4" Grid.Row="0" Text="Stream" FontSize="16" Foreground="Gray" />
        <TextBlock Grid.Column="6" Grid.Row="0" Text="Pos" FontSize="16" Foreground="Gray" />
        <TextBlock Grid.Column="8" Grid.Row="0" Text="Parameter" FontSize="16" Foreground="Gray" />
        <TextBox x:Name="NameText" Grid.Column="0" Grid.Row="1" />
        <ComboBox x:Name="SizeBox" Grid.Column="2" Grid.Row="1" SelectedIndex="0" HorizontalAlignment="Stretch">
            <ComboBoxItem Content="1" Tag="1" />
            <ComboBoxItem Content="2" Tag="2" />
            <ComboBoxItem Content="3" Tag="3" />
            <ComboBoxItem Content="4" Tag="4" />
            <ComboBoxItem Content="5" Tag="5" />
            <ComboBoxItem Content="6" Tag="6" />
            <ComboBoxItem Content="7" Tag="7" />
            <ComboBoxItem Content="8" Tag="8" />
            <ComboBoxItem Content="9" Tag="9" />
        </ComboBox>
        <ComboBox x:Name="StreamBox" Grid.Column="4" Grid.Row="1" HorizontalAlignment="Stretch" SelectionChanged="StreamBox_SelectionChanged" />
        <TextBox x:Name="PosText" Grid.Column="6" Grid.Row="1" InputScope="Number" TextChanging="NumText_TextChanging" />
        <ComboBox x:Name="ParameterBox" Grid.Column="8" Grid.Row="1" HorizontalAlignment="Stretch" SelectionChanged="ParameterBox_SelectionChanged" />
        <TextBlock x:Name="DescText" Grid.Row="3" Grid.Column="0" Grid.ColumnSpan="8" FontSize="12" Foreground="Gray" TextTrimming="CharacterEllipsis" />
    </Grid>
</UserControl>
